# fylr

[![release](https://github.com/programmfabrik/fylr/actions/workflows/release.yml/badge.svg?branch=production)](https://github.com/programmfabrik/fylr/actions/workflows/release.yml)
[![unit-and-apitests](https://github.com/programmfabrik/fylr/actions/workflows/tests.yml/badge.svg)](https://github.com/programmfabrik/fylr/actions/workflows/tests.yml)

## Getting started

### Tooling

**Go 1.21** is required as the minimum supported version for the project. Follow the [standard Golang instructions](https://golang.org/doc/install). The end result is, you should be able to run the following commands without issues

```bash
go version
go mod
```

**Nodejs 14.15.4. LTS** is _recommended_ as the minimum supported version for the project. The suggested setup is through
[NVM, Nodejs Version Manager](https://github.com/nvm-sh/nvm)

<details>
    <summary>3. [Optional] Python 2</summary>
<p>

**Python2** is required as the currently compatible version for integrating with the [EasyDB Webfrontend](https://github.com/programmfabrik/easydb-webfrontend). Follow the link for further installation instructions

</p>
</details>

### Project dependencies

fylr needs a number of compile-time dependencies installed on your system.

You can find an up-to-date list of dependency packages at `docker/Dockerfile`.
The package names there are Alpine Linux packages. You may need to find the
correct package name for the distribution that you are using on your dev machine.

Also, install SASS (v1) via NPM using:

```bash
npm i -g coffeescript@1.12.7 sass
```

### Clone project

```bash
git clone git@github.com:programmfabrik/fylr.git
cd fylr/
git submodule update --init --recursive
```

### Clone plugins
For local API testing, you also need the plugins for fylr and easydb:

```bash
git clone git@github.com:programmfabrik/easydb-plugins
git clone git@github.com:programmfabrik/fylr-plugin-example
git clone git@github.com:programmfabrik/fylr-plugin-formula-columns
git clone git@github.com:programmfabrik/fylr-plugin-server-pdf
cd easydb-plugins
git submodule update --init --recursive
cd ../fylr-plugin-example
make build
cd ../fylr-plugin-formula-columns
make build
cd ../fylr-plugin-server-pdf
make build
```

### Build project

If all dependencies are in place, building is as easy as

```bash
make
```

This will generate the `fylr` binary.

### Start server

#### Server configuration
fylr comes with a working configuration out-of-the box.

Start by creating an empty `fylr.yml` file.
Defaults are then read from `internal/global/fylr.default.yml` and can be
overridden with values from your `fylr.yml`.

Configuration options are described in detail in `fylr.example.yml`.

If you want to run your local development without root permissions, it is
advisable to override the web frontend port in your `fylr.yml` as follows:

```yaml
fylr+:
  externalURL: "http://localhost:8099"
  services+:
    webapp+:
      addr: ":8099"
```

For local API testing, you may want to add the following overrides:

```yaml
fylr+:
  [...]
  allowpurge: true                 # allow purging of data via API for testing
                                   # (see also below under db+)

                                   # set up an encryption key for related tests
  encryptionKey: "12345678901234567890123456789012"

  services+:
    api+:
                                   # some tests need a webDAV hotfolder
      webDAVHotfolderPath: "/tmp/hotfolder"

  db+:
    init+:
      config+:
        system+:
          config+:
            purge:                 # allow purging of data via API for testing
              allow_purge: true
              purge_storage: true

  plugin:                         # install available plugins for testing
    paths:
      - "../easydb-plugins"
      - "../fylr-plugin-example/build"
      - "../fylr-plugin-formula-columns/build"
      - "../fylr-plugin-server-pdf/build"
```

It is recommended to look at the configuration of the CI/CD at
`docker/apitest/fylr.postgres.yml` to ensure you have an up-to-date config.

An example config is also available at:

https://raw.githubusercontent.com/programmfabrik/fylr-gitbook/main/_assets/fylr.yml

Please note the following:

1. `execserver.*.commands` might require updating depending on the dependencies you ended up installing
2. `apitest.store.config_email` will require your attention
3. check at least the lines with the string `EXAMPLE`, e.g. https://EXAMPLE.COM could be placed with http://localhost.
4. Look into `cmd/fylr/fylr.example.yml` for inspiration.
5. If not explicitly specified in your fylr.yml, defaults will be used from `internal/global/fylr.default.yml`.

_The name of fylr.yml is significant, though it can be changed by passing `-c` option to the `fylr` binary._

On some platforms (e.g. Fedora), some CLI commands may be named differently than
what is configured in our defaul config (e.g. chromium is called chromium-browser).
You can find override settings in fylr.example.yml.

#### Server dependencies

For local development, you need a working PostgreSQL and OpenSearch
installation.

If you want to use a docker-compose file for that, have a look at
`docker/docker-compose.cicd.yml` for a good starting point.

You need to configure your databases with fylr. For details on that, refer to
`fylr.example.yml`.

If you need email sending/receiving capabilities for local testing, have a look
at https://github.com/sj26/mailcatcher - a very simple local fake mail server
with a nice GUI. It is also available via Docker.

#### Run

```bash
make server
```

### Test server
The API tests need a configuration file in order to work.

An example configuration can look like this:

```yaml
apitest:
  server: http://localhost:8080/api/v1
  db-name: fylr-apitest
  store:
    run_failing_tests: 1
    oauth2_client: fylr-web-frontend
    send_email: "your.name+fylr1@programmfabrik.de"
    send_email2: "your.name+fylr2@programmfabrik.de"
    send_email3: "your.name+fylr3@programmfabrik.de"
    send_email_confirm: "your-name-fylr-confirm@programmfabrik.de"
    config_email:
      from: "fylr-apitest@programmfabrik.de"
      admin_emails:
      - email: "your.name+fylr-admin@programmfabrik.de"
      server_addr: "localhost:1025"
  oauth_client:
    fylr-web-frontend:
      endpoint:
        auth_url: http://localhost:8099/api/oauth2/auth
        token_url: http://localhost:8099/api/oauth2/token
      secret: foo
```

For local development, you should store the configuration in `apitest.yml`.

It is recommended to look at the configuration of the CI/CD at
`docker/apitest/apitest.postgres.yml` to ensure you have an up-to-date config.

Export this environment variable, it's best to store it in your shell profile.

```bash
export APITEST_PARAMS="-c apitest.yml --stop-on-fail"
```

Be sure that the plugins are installed and configured correctly (see previously),
otherwise the API tests won't pass.

Note that API tests need a PostgreSQL datastore, so you should configure one
in your `fylr.yml` accordingly. Running tests using Sqlite may result in test
failures.

At this point, you can try running `apitest` and brew yourself some good coffee.

```bash
make apitest
```

<details>
<summary>Troubleshooting Common Issues</summary>
<p>

1. If you receive this error

```log
ERRO[0000] [ 0] error sending request: Could not do http request: Get "/session": unsupported protocol scheme ""
```

Then you forgot to set the environment variable `APITEST_PARAMS` mentioned in the step before this.

</p>
</details>
&nbsp;


### [OPTIONAL] Access the web frontend

If you followed the optional step in [Getting Started](#getting-started) then you should
have cloned the [Web frontend Repo](https://github.com/programmfabrik/easydb-webfrontend)

Make sure you specify the location of the repo in `fylr.yml` configuration file, by setting `server.webfrontend`.

Follow the instructions in Web frontend repo, then restart fylr server again by exiting the process and running `make server` once more.

If you used the default URL configurations then you should be able to load [http://localhost:8080](http://localhost:8080)

Default login credentials are:
- **Username**: *root*
- **Password**: *admin*

---

## Docker Container Setup

You can check the docker setup in the [docker directory](docker/). This should also be your current working directory so for this type of setup. `cd docker`

### Build The Image

To build the image, all you have to do is make sure you're inside [docker directory](docker/) and then run

```bash
make container
```
<details>
<summary>[Optional] Custom docker build arguments </summary>
<p>

```bash
export DOCKER_CONTAINER_BUILD_ARGUMENTS='--label "git_commit_sha=${{ github.sha }}" --label "maintainer=Leon Steinhäuser <leon.steinhaeuser@fylr.io>"'
```

</p>
</details>
&nbsp;

## Releases and Commits

* Detailed information about *releases* can be found on the page [Releases](https://github.com/programmfabrik/fylr/wiki/Releases).
* Detailed information about the *git commit* syntax can be found on page [Commits](https://github.com/programmfabrik/fylr/wiki/Commits).
